using System;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <see cref="IFileMatcher"/> defines the contract to match a certain file
    /// (in order for an attaching <see cref="IFileProcessor"/> to process the file).
    ///
    /// <contract-constructor>
    /// All implementation should have a constructor with a namespace and a prefix
    /// as arguments in order to reflect from configuration file. This requirement
    /// can be dropped if it's not necessary to use this way.
    /// </contract-constructor>
    ///
    /// <thread-safety>
    /// Implementations are advised to be stateless and thread-safe.
    /// But it's not a requirement.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public interface IFileMatcher
    {
        /// <summary>
        /// This is used to determine if the file can/will be processed by the processor
        /// that this matcher is associated with.
        /// </summary>
        ///
        /// <param name="file">the file to match.</param>
        /// <returns>whether the file matches.</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null.</exception>
        /// <exception cref="IOException">if io operation fails.</exception>
        /// <exception cref="FileMatchingException">if error occurs.</exception>
        bool Match(FileInfo file);
    }
}