using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileNameMatcher"/> is a template class that handles filename oriented matching logic.
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public abstract class FileNameMatcher : IFileMatcher
    {
        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected FileNameMatcher()
        {
        }

        /// <summary>
        /// It extracts the filename and calls the template method.
        /// </summary>
        ///
        /// <param name="file">the file to match</param>
        /// <returns>whether the file matches</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="FileMatchingException">if error occurs</exception>
        public bool Match(FileInfo file)
        {
            // verfiy argument
            FileStatisticsHelper.CheckNull(file, "file");

            try
            {
                return Match(file.Name);
            }
            catch(Exception e)
            {
                throw new FileMatchingException("Failed to match for " + e.Message, e);
            }
        }

        /// <summary>
        /// Template method to match a filename.
        /// </summary>
        ///
        /// <param name="filename">the filename to match</param>
        /// <returns>whether the filename matches</returns>
        ///
        /// <exception cref="FileMatchingException">if error occurs</exception>
        protected abstract bool Match(string filename);
    }
}
