using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for <see cref="AnyFileMatcher"/> class.
    /// It mainly tests the following:
    /// constructor: AnyFileMatcher(), AnyFileMatcher(string, string)
    /// member: Match(FileInfo)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class AnyFileMatcherUnitTest
    {
        /// <summary>
        /// Test ctor AnyFileMatcher(), simple constructor,
        /// isntance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new AnyFileMatcher(), "Failed to create instance of AnyFileMatcher.");
        }

        /// <summary>
        /// Test ctor AnyFileMatcher(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NamespaceIsNull()
        {
            new AnyFileMatcher(null, "prefix");
        }

        /// <summary>
        /// Test ctor AnyFileMatcher(string nameSpace, string prefix), with nameSpace is empty,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NamespaceIsEmpty()
        {
            new AnyFileMatcher("  ", "prefix");
        }

        /// <summary>
        /// Test ctor AnyFileMatcher(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PrefixIsNull()
        {
            new AnyFileMatcher("TopCoder", null);
        }

        /// <summary>
        /// Test ctor AnyFileMatcher(string nameSpace, string prefix), with prefix is emtpy, allowed.
        /// </summary>
        [Test]
        public void TestCtor2_PrefixIsEmpty()
        {
            Assert.IsNotNull(new AnyFileMatcher("TopCoder", "  "),
                "Failed to create instance of AnyFileMatcher.");
        }

        /// <summary>
        /// Test ctor AnyFileMatcher(string nameSpace, string prefix), with both arguments are non-emtpy,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Assert.IsNotNull(new AnyFileMatcher("TopCoder", "prefix"),
                "Failed to create instance of AnyFileMatcher.");
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestMatch_FileIsNull()
        {
            IFileMatcher matcher = new AnyFileMatcher();
            matcher.Match(null);
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is not null, true is expected.
        /// </summary>
        [Test]
        public void TestMatch()
        {
            IFileMatcher matcher = new AnyFileMatcher();
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/Usage.txt")),
                "True is always expected for Match of AnyFileMatcher.");
        }
    }
}