using System;
using NUnit.Framework;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Unit Test for <see cref="Metric"/> class.
    /// <see cref="Metric"/> class is simple, just need test the following:
    /// constructor: Metric(string, string, string)
    /// properties: Name, DisplayName, Description
    /// members: Equals, GetHashCode.
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class MetricUnitTest
    {
        /// <summary>
        /// Represents the name of the meric, used to construct the metric.
        /// </summary>
        private string name = "tcs";

        /// <summary>
        /// Represents the display name of the meric, used to construct the metric.
        /// </summary>
        private string displayName = "topcoder";

        /// <summary>
        /// Represents the description of the meric, used to construct the metric.
        /// </summary>
        private string description = "topcoder software";

        /// <summary>
        /// Represents an instance of Metric, used to test it's members.
        /// </summary>
        private Metric metric;

        /// <summary>
        /// Set up for each test,
        /// construct an instance of Metric.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            metric = new Metric(name, displayName, description);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with name is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor_NameIsNull()
        {
            new Metric(null, displayName, description);
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with name is emtpy string, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor_NameIsEmpty()
        {
            new Metric("   ", displayName, description);
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with displayName is Null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor_DisplayNameIsNull()
        {
            new Metric(name, null, description);
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with displayName is empty string, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor_DisplayNameIsEmpty()
        {
            new Metric(name, "  ", description);
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with description is Null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor_DescriptionIsNull()
        {
            new Metric(name, displayName, null);
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with description is empty string, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor_DescriptionIsEmpty()
        {
            new Metric(name, displayName, "   ");
        }

        /// <summary>
        /// Test ctor Metric(string name, string displayName, string description),
        /// with all arguments are valid, instance should be created.
        /// </summary>
        [Test]
        public void TestCtor()
        {
            Assert.IsNotNull(metric, "Failed to create instance of Metric.");
        }

        /// <summary>
        /// Test property Name Getter, it should return the same name as the ctor's.
        /// </summary>
        [Test]
        public void TestName_Getter()
        {
            Assert.AreEqual(name, metric.Name, "Failed to get the correct name.");
        }

        /// <summary>
        /// Test property DisplayName Getter, it should return the same display name as the ctor's.
        /// </summary>
        [Test]
        public void TestDisplayName_Getter()
        {
            Assert.AreEqual(displayName, metric.DisplayName, "Failed to get the correct display name.");
        }

        /// <summary>
        /// Test property Description Getter, it should return the same description as the ctor's.
        /// </summary>
        [Test]
        public void TestDescription_Getter()
        {
            Assert.AreEqual(description, metric.Description,
                "Failed to get the correct display name.");
        }

        /// <summary>
        /// Test Equals(object obj), the metric with the same name should be equality.
        /// </summary>
        [Test]
        public void TestEquals1()
        {
            Metric metric2 = new Metric(name, displayName, description);

            Assert.IsTrue(metric.Equals(metric2), "The Equals is not implemented correctly.");
        }

        /// <summary>
        /// Test Equals(object obj), with the name of metric is not the same, false is expected.
        /// </summary>
        [Test]
        public void TestEquals2()
        {
            Metric metric2 = new Metric("new", displayName, description);
            Assert.IsFalse(metric.Equals(metric2), "The Equals is not implemented correctly.");
        }

        /// <summary>
        /// Test Equals(object obj), with the obj is not metric type, false is expected.
        /// </summary>
        [Test]
        public void TestEquals_NotMetricType()
        {
            Assert.IsFalse(metric.Equals(new object()), "The Equals is not implemented correctly.");
        }

        /// <summary>
        /// Test GetHashCode(), the returned hash code should be equal to the Name's hashcode.
        /// </summary>
        [Test]
        public void TestGetHashCode1()
        {
            Assert.AreEqual(name.GetHashCode(), metric.GetHashCode(),
                "The Equals is not implemented correctly.");
        }

        /// <summary>
        /// Test GetHashCode(), the returned hash code of metric with different name should be not equal.
        /// </summary>
        [Test]
        public void TestGetHashCode2()
        {
            Metric metric2 = new Metric("new", displayName, description);
            Assert.IsFalse(metric2.GetHashCode() == metric.GetHashCode(),
                "The Equals is not implemented correctly.");
        }
    }
}